#!/bin/bash
# Copyright (c) 1992-2016 ZAO "MCST". All rights reserved.
#
# @(#) $Id: event2ptrace.sh,v 1.9 2017-05-27 20:48:14 breger Exp $
#
# Script to convert "dprof" profiling results
# into ptrace format for using with "ldis" disassembler.
# Can be used when "-s" or "-t" options of dprof were enabled.
#
# Two modes are supported:
#
# * Results of "dprof" or "dprof -d" (profiling timer), Usage:
#  dprof -o IP.timer.all test
#  dprof2ptrace.sh IP.timer.all > test.ptrace
#
# Warning: script will filter output, and only lines starting with
# "[pid] 0000address", e.g. 
# "[5422] 0000000000002cb0"
# will be parsed
#
# * Results of "dprof -e" (hardware events), Usage: 
#  dprof -o IP.14.all -e 2:14:10000 test
#  event2ptrace.sh IP.14.all > test.ptrace
#
# Options to 
#  ldis -P test                # to get list of functions sorted by cost
#  ldis -J test function_name  # to get assembly listing of "function_name"
#
# Warning: script will filter output, and only lines starting with
# "[pid] event EVENT_NAME (chan) at 0000address" e.g.
# "[5427] event BUB_E2 (2) at 0000000000002cb0"
# will be parsed
#
# TODO: detect dprof -P; parse proc/pid/maps?


scriptname=`basename $0`
# Формируем такой же заголовок, как и lcsim (Bug 84824)
echo "# CYCLE # by $scriptname `date -Is`"
echo "# Address Width Executed Trapped Consumed NOPs R-stalls FPU-stalls CODE-stalls SPILL-stalls MOVA-stalls MMU-stalls Unloaded BRANCH-stalls Wait-stalls Trap-stalls"

case "$scriptname" in
    dprof2*) dprof_timer_events=1 
awk '
/^\[[0-9]+\] 0[0-9a-fA-F]+/ {
    count[$2]++;
}
END {
    for (i in count) {
        print "0x" i " 4 " count[i] " 0 " count[i] " 0 0 0 0 0 0 0 0 0 0 0 "
    }
}
' $@ | sort
    ;;
    event2*) dprof_hw_events=1 
awk '
/^\[[0-9]+\] event \w+ \(\w*:?\w+\) at 0[0-9a-fA-F]+/ {
    count[$6]++;
}
END {
    for (i in count) {
        print "0x" i " 4 " count[i] " 0 " count[i] " 0 0 0 0 0 0 0 0 0 0 0 "
    }
}
' $@ | sort
    ;;
esac

