#!/bin/bash
# Copyright (c) 2010 ZAO "MCST". All rights reserved.
#
# @(#) $Id: extractor.sh,v 1.4 2015-11-18 20:03:20 breger Exp $
#
# Script to convert ptrace files after event2ptrace.sh and dprof2ptrace.sh
#  scripts into separate ptraces for dynamic libraries.
#
# Usage: 
#  dprof -o IP.14.all -e 2:14:10000 test.dynamic
#  event2ptrace.sh IP.14.all > test.dynamic.ptrace
#  extractor.sh test.dynamic             # will create or use $1.dynamic_libs file
#  ldis -P test.dynamic                  # Dynamic functions will be counted as "Unknown"
#  ldis -J test.dynamic function_name    # To view listing of function from main
#  ldd test.dynamic # 
#  cp /path/to/library/liblibrary.so .   # where liblibrary.so is the name of dynamic library; 
#  #/path/to/library is printed by `ldd` utility 
#  ldis -P liblibrary.so         # Only events from liblibrary.so is shown
#  ldis -J liblibrary.so lib_function1   # Listing of lirary function with name "lib_function1"
#
# Warning! If there is no $1.dynamic_libs file, this script must be started on e2k machine (native)
#  If the binary or libraries are changed, delete $1.dynamic_libs file.
#  This script may overwrite any library_name.ptrace file in current directory
#
# Output of this script (stdout):
#  Line "Updating $1.dynamic_libs file" to get list of libraries and their load addresses
#  a line for each library, that have nonzero events logged:
#  "library_name event_count", e.g.
#  "libc.so.6 305"
#

if [[ -f "$1" ]]; then
if [[ -f "$1.ptrace" ]]; then

# Try to open $1.dynamic_libs. If there is a recent file, don't run ldd
if [ "(" "!" "(" -f "$1.dynamic_libs" ")" ")" -o "(" $1 -nt "$1.dynamic_libs" ")" ]; then
 echo Updating $1.dynamic_libs file
 ldd $1 | sort -k 4 |
 awk '/=>/ { match($4,/\(([0-9a-fx]+)\)/,a); print n " " addr " " a[1]; t=split($1,ta,/\//); n=ta[t]; addr=a[1]} END {print n " " addr " 0xffffffffffffffff"}' > $1.dynamic_libs

fi

awk --non-decimal-data '
BEGIN {
  while (getline <"'$1.dynamic_libs'")
  {
    name=$1;
    addr_b[name]=$2+0;
    addr_e[name]=$3+0;
    if( addr_e[name] != 0 )
    {
      printf "#\n#\n" >name ".ptrace";
    }
  }
}
{
  ip=$1+0;
  for (n in addr_b)
  {
    if ((ip>=addr_b[n]) && (ip<addr_e[n]))
    {
      c[n]+=$3;
      printf "0x%x ",ip-addr_b[n] >n ".ptrace";
      print $2 " " $3 " " $4 " " $5 " " $6 " " $7 " " $8 " " $9 " " $10 " " $11 " " $12 " " $13 " " $14 " " $15 " " $16 >>n ".ptrace"
    }
  }
} 
END {
  for (n in c)
  {
    print n " " c[n]
  }
}' $1.ptrace

else 
 echo "File " $1".ptrace not found"
 echo " Usage: ./extractor.sh file"
fi

else 
 if [ ! -z $1 ]; then 
   echo "File " $1" not found"
 fi
 echo " Usage: ./extractor.sh file"
 echo "File 'file' and 'file.ptrace' must exist"
fi

